@@ -238,6 +238,19 @@ class TikTokApi:
 
         query = {"verifyFp": verify_fp, "did": did, "_signature": signature}
         url = "{}&{}".format(kwargs["url"], urlencode(query))
+
+        h = requests.head(
+            url,
+            headers= {
+                "x-secsdk-csrf-version": "1.2.5",
+                "x-secsdk-csrf-request": "1"
+            },
+            proxies=self.__format_proxy(proxy),
+            **self.requests_extra_kwargs)
+        csrf_session_id = h.cookies["csrf_session_id"]
+        csrf_token = h.headers["X-Ware-Csrf-Token"].split(",")[1]
+        kwargs["csrf_session_id"] = csrf_session_id
+
         r = requests.get(
             url,
             headers={
@@ -255,9 +268,7 @@ class TikTokApi:
                 "sec-fetch-site": "same-site",
                 "sec-gpc": "1",
                 "user-agent": userAgent,
-                "x-secsdk-csrf-token": "".join(
-                    random.choice(string.ascii_uppercase + string.ascii_lowercase)
-                    for i in range(92))
+                "x-secsdk-csrf-token": csrf_token
             },
             cookies=self.get_cookies(**kwargs),
             proxies=self.__format_proxy(proxy),
@@ -309,6 +320,7 @@ class TikTokApi:
             return {
                 "tt_webid": did,
                 "tt_webid_v2": did,
+                "csrf_session_id": kwargs.get("csrf_session_id"),
                 "tt_csrf_token": "".join(
                     random.choice(string.ascii_uppercase + string.ascii_lowercase)
                     for i in range(16)
@@ -319,6 +331,7 @@ class TikTokApi:
             return {
                 "tt_webid": did,
                 "tt_webid_v2": did,
+                "csrf_session_id": kwargs.get("csrf_session_id"),
                 "tt_csrf_token": "".join(
                     random.choice(string.ascii_uppercase + string.ascii_lowercase)
                     for i in range(16)
@@ -1621,7 +1634,7 @@ class TikTokApi:
             raise Exception(
                 "Retrieving the user secUid failed. Likely due to TikTok wanting captcha validation. Try to use a proxy."
             )
-    
+
     @staticmethod
     def generate_did():
         """Generates a valid did for other methods. Pass this as the custom_did field to download videos"""
